@@ -1,23 +1,19 @@
 """Parameter class."""
-from __future__ import division
 
-from collections import OrderedDict
 from copy import deepcopy
-import importlib
 import json
-import warnings
 
 from asteval import Interpreter, get_ast_names, valid_symbol_name
-from numpy import arcsin, array, cos, inf, isclose, nan, sin, sqrt
+from numpy import arcsin, array, cos, inf, isclose, sin, sqrt
 import scipy.special
-import uncertainties
 
 from .jsonutils import decode4js, encode4js
+from .lineshapes import tiny
 from .printfuncs import params_html_table
 
-SCIPY_FUNCTIONS = {'gamfcn': scipy.special.gamma}
-for name in ('erf', 'erfc', 'wofz'):
-    SCIPY_FUNCTIONS[name] = getattr(scipy.special, name)
+SCIPY_FUNCTIONS = {'gamfcn': scipy.special.gamma, 'loggammafcn': scipy.special.loggamma, 'betalnfnc': scipy.special.betaln}
+for fnc_name in ('erf', 'erfc', 'wofz'):
+    SCIPY_FUNCTIONS[fnc_name] = getattr(scipy.special, fnc_name)
 
 
 def check_ast_errors(expr_eval):
@@ -26,11 +22,13 @@ def check_ast_errors(expr_eval):
         expr_eval.raise_exception(None)
 
 
-class Parameters(OrderedDict):
-    """An ordered dictionary of all the Parameter objects required to
-    specify a fit model. All minimization and Model fitting routines in
-    lmfit will use exactly one Parameters object, typically given as the
-    first argument to the objective function.
+class Parameters(dict):
+    """A dictionary of Parameter objects.
+
+    It should contain all Parameter objects that are required to specify
+    a fit model. All minimization and Model fitting routines in lmfit will
+    use exactly one Parameters object, typically given as the first
+    argument to the objective function.
 
     All keys of a Parameters() instance must be strings and valid Python
     symbol names, so that the name must match ``[a-z_][a-z0-9_]*`` and
@@ -38,7 +36,7 @@ class Parameters(OrderedDict):
 
     All values of a Parameters() instance must be Parameter objects.
 
-    A Parameters() instance includes an asteval interpreter used for
+    A Parameters() instance includes an `asteval` Interpreter used for
     evaluation of constrained Parameters.
 
     Parameters() support copying and pickling, and have methods to convert
@@ -46,27 +44,18 @@ class Parameters(OrderedDict):
 
     """
 
-    def __init__(self, asteval=None, usersyms=None, *args, **kwds):
+    def __init__(self, usersyms=None):
         """
         Arguments
         ---------
-        asteval : :class:`asteval.Interpreter`, optional
-            Instance of the asteval Interpreter to use for constraint
-            expressions. If None, a new interpreter will be created.
-            Warning: *deprecated, use usersyms if possible*
-        usersyms : dictionary of symbols to add to the
-            :class:`asteval.Interpreter`.
-        *args : optional
-            Arguments.
-        **kwds : optional
-            Keyword arguments.
+        usersyms : dict, optional
+            Dictionary of symbols to add to the
+            :class:`asteval.Interpreter` (default is None).
 
         """
-        super(Parameters, self).__init__(self)
+        super().__init__(self)
 
-        self._asteval = asteval
-        if self._asteval is None:
-            self._asteval = Interpreter()
+        self._asteval = Interpreter()
 
         _syms = {}
         _syms.update(SCIPY_FUNCTIONS)
@@ -82,7 +71,7 @@ class Parameters(OrderedDict):
     def update(self, other):
         """Update values and symbols with another Parameters object."""
         if not isinstance(other, Parameters):
-            raise ValueError("'%s' is not a Parameters object" % other)
+            raise ValueError(f"'{other}' is not a Parameters object")
         self.add_many(*other.values())
         for sym in other._asteval.user_defined_symbols():
             self._asteval.symtable[sym] = other._asteval.symtable[sym]
@@ -95,15 +84,21 @@ class Parameters(OrderedDict):
     def __deepcopy__(self, memo):
         """Implementation of Parameters.deepcopy().
 
-        The method needs to make sure that asteval is available and that all
-        individual Parameter objects are copied.
+        The method needs to make sure that `asteval` is available and that
+        all individual Parameter objects are copied.
 
         """
-        _pars = Parameters(asteval=None)
+        _pars = self.__class__()
 
         # find the symbols that were added by users, not during construction
-        unique_symbols = {key: self._asteval.symtable[key]
-                          for key in self._asteval.user_defined_symbols()}
+        unique_symbols = {}
+        for key in self._asteval.user_defined_symbols():
+            try:
+                val = deepcopy(self._asteval.symtable[key])
+                unique_symbols[key] = val
+            except (TypeError, ValueError):
+                unique_symbols[key] = self._asteval.symtable[key]
+
         _pars._asteval.symtable.update(unique_symbols)
 
         # we're just about to add a lot of Parameter objects to the newly
@@ -117,10 +112,10 @@ class Parameters(OrderedDict):
                 param.vary = par.vary
                 param.brute_step = par.brute_step
                 param.stderr = par.stderr
-                param.correl = par.correl
+                param.correl = deepcopy(par.correl)
                 param.init_value = par.init_value
                 param.expr = par.expr
-                param.user_data = par.user_data
+                param.user_data = deepcopy(par.user_data)
                 parameter_list.append(param)
 
         _pars.add_many(*parameter_list)
@@ -128,13 +123,12 @@ class Parameters(OrderedDict):
         return _pars
 
     def __setitem__(self, key, par):
-        """TODO: add magic method docstring."""
-        if key not in self:
-            if not valid_symbol_name(key):
-                raise KeyError("'%s' is not a valid Parameters name" % key)
+        """Set items of Parameters object."""
+        if key not in self and not valid_symbol_name(key):
+            raise KeyError(f"'{key}' is not a valid Parameters name")
         if par is not None and not isinstance(par, Parameter):
-            raise ValueError("'%s' is not a Parameter" % par)
-        OrderedDict.__setitem__(self, key, par)
+            raise ValueError(f"'{par}' is not a Parameter")
+        dict.__setitem__(self, key, par)
         par.name = key
         par._expr_eval = self._asteval
         self._asteval.symtable[key] = par.value
@@ -142,7 +136,7 @@ class Parameters(OrderedDict):
     def __add__(self, other):
         """Add Parameters objects."""
         if not isinstance(other, Parameters):
-            raise ValueError("'%s' is not a Parameters object" % other)
+            raise ValueError(f"'{other}' is not a Parameters object")
         out = deepcopy(self)
         out.add_many(*other.values())
         for sym in other._asteval.user_defined_symbols():
@@ -178,9 +172,9 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         state : dict
-            state['unique_symbols'] is a dictionary containing symbols that
-            need to be injected into _asteval.symtable
-            state['params'] is a list of Parameter instances to be added
+            state['unique_symbols'] is a dictionary containing symbols
+            that need to be injected into `_asteval.symtable`.
+            state['params'] is a list of Parameter instances to be added.
 
         """
         # first update the Interpreter symbol table. This needs to be done
@@ -191,44 +185,44 @@ class Parameters(OrderedDict):
         # an Error. Another way of doing this would be to remove all the expr
         # from the Parameter instances before they get added, then to restore
         # them.
-        # self._asteval.symtable.update(state['unique_symbols'])
 
         symtab = self._asteval.symtable
         for key, val in state['unique_symbols'].items():
             if key not in symtab:
-                if isinstance(val, dict):
-                    value = val.get('__name__', None)
-                    symname = val.get('__name__', None)
-                    importer = val.get('importer', None)
-                    if value is None and symname is not None and importer is not None:
-                        _mod = importlib.import_module(importer)
-                        value = getattr(_mod, symname, None)
-                    if value is not None:
-                        symtab[key] = value
-                else:
-                    symtab[key] = val
+                symtab[key] = val
+
         # then add all the parameters
         self.add_many(*state['params'])
 
+    def __repr__(self):
+        """__repr__ from OrderedDict."""
+        if not self:
+            return f'{self.__class__.__name__}()'
+        return f'{self.__class__.__name__}({list(self.items())!r})'
+
     def eval(self, expr):
-        """Evaluate a statement using the asteval Interpreter.
+        """Evaluate a statement using the `asteval` Interpreter.
 
         Parameters
         ----------
-        expr : string
+        expr : str
             An expression containing parameter names and other symbols
-            recognizable by the asteval Interpreter.
+            recognizable by the `asteval` Interpreter.
 
         Returns
         -------
-           The result of the expression.
+        float
+            The result of evaluating the expression.
 
         """
         return self._asteval.eval(expr)
 
     def update_constraints(self):
-        """Update all constrained parameters, checking that dependencies are
-        evaluated as needed."""
+        """Update all constrained parameters.
+
+        This method ensures that dependencies are evaluated as needed.
+
+        """
         requires_update = {name for name, par in self.items() if par._expr is
                            not None}
         updated_tracker = set(requires_update)
@@ -236,8 +230,8 @@ class Parameters(OrderedDict):
         def _update_param(name):
             """Update a parameter value, including setting bounds.
 
-            For a constrained parameter (one with an `expr` defined),
-            this first updates (recursively) all parameters on which the
+            For a constrained parameter (one with an `expr` defined), this
+            first updates (recursively) all parameters on which the
             parameter depends (using the 'deps' field).
 
             """
@@ -259,20 +253,20 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         oneline : bool, optional
-            If True prints a one-line parameters representation (default is
-            False).
+            If True prints a one-line parameters representation (default
+            is False).
 
         Returns
         -------
         s: str
-           Parameters representation.
+            Parameters representation.
 
         """
         if oneline:
-            return super(Parameters, self).__repr__()
+            return self.__repr__()
         s = "Parameters({\n"
         for key in self.keys():
-            s += "    '%s': %s, \n" % (key, self[key])
+            s += f"    '{key}': {self[key]}, \n"
         s += "    })\n"
         return s
 
@@ -284,17 +278,21 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         oneline : bool, optional
-            If True prints a one-line parameters representation (default is
-            False).
+            If True prints a one-line parameters representation (default
+            is False).
         colwidth : int, optional
-            Column width for all columns specified in :attr:`columns`.
+            Column width for all columns specified in `columns` (default
+            is 8).
         precision : int, optional
-            Number of digits to be printed after floating point.
+            Number of digits to be printed after floating point (default
+            is 4).
         fmt : {'g', 'e', 'f'}, optional
-            Single-character numeric formatter. Valid values are: 'f' floating
-            point, 'g' floating point and exponential, or 'e' exponential.
+            Single-character numeric formatter. Valid values are: `'g'`
+            floating point and exponential (default), `'e'` exponential,
+            or `'f'` floating point.
         columns : :obj:`list` of :obj:`str`, optional
-            List of :class:`Parameter` attribute names to print.
+            List of :class:`Parameter` attribute names to print (default
+            is to show all attributes).
 
         """
         if oneline:
@@ -309,7 +307,7 @@ class Parameters(OrderedDict):
         otherstyles = dict(name='{name:<{name_len}} ', stderr='{stderr!s:>{n}}',
                            vary='{vary!s:>{n}}', expr='{expr!s:>{n}}',
                            brute_step='{brute_step!s:>{n}}')
-        line = ' '.join([otherstyles.get(k, numstyle % k) for k in allcols])
+        line = ' '.join(otherstyles.get(k, numstyle % k) for k in allcols)
         for name, values in sorted(self.items()):
             pvalues = {k: getattr(values, k) for k in columns}
             pvalues['name'] = name
@@ -324,30 +322,79 @@ class Parameters(OrderedDict):
                               f=fmt, **pvalues))
 
     def _repr_html_(self):
-        """Returns a HTML representation of parameters data."""
+        """Return a HTML representation of parameters data."""
         return params_html_table(self)
 
+    def set(self, **kws):
+        """Set Parameter values and other attributes.
+
+        Parameters
+        ----------
+        **kws : optional
+            Parameter names and initial values or dictionaries of
+                 values and attributes.
+
+        Returns
+        -------
+        None
+
+        Notes
+        -----
+        1. keyword arguments will be used to create parameter names.
+        2. values can either be numbers (floats or integers) to set the
+           parameter value, or can be dictionaries with any of the following
+           keywords: ``value``, ``vary``, ``min``, ``max``, ``expr``,
+           ``brute_step``, or ``is_init_value`` to set those parameter attributes.
+        3. for each parameter,  ``is_init_value`` controls whether to set
+           ``init_value`` when setting ``value``, and defaults to True.
+
+        Examples
+        --------
+        >>> params = Parameters()
+        >>> params.add('xvar', value=0.50, min=0, max=1)
+        >>> params.add('yvar', expr='1.0 - xvar')
+        >>> params.set(xvar=0.80, zvar={'value':3, 'min':0})
+
+        """
+        for name, val in kws.items():
+            if name not in self:
+                self.__setitem__(name, Parameter(value=-inf, name=name,
+                                                 vary=True, min=-inf, max=inf,
+                                                 expr=None, brute_step=None))
+            par = self.__getitem__(name)
+            if isinstance(val, (float, int)):
+                val = {'value': val}
+            if 'is_init_value' not in val:
+                val['is_init_value'] = True
+            par.set(**val)
+
     def add(self, name, value=None, vary=True, min=-inf, max=inf, expr=None,
             brute_step=None):
         """Add a Parameter.
 
         Parameters
         ----------
-        name : str
-            Name of parameter.  Must match ``[a-z_][a-z0-9_]*`` and cannot be
-            a Python reserved word.
+        name : str or Parameter
+            If ``name`` refers to a Parameter object it will be added directly
+            to the Parameters instance, otherwise a new Parameter object with name
+            ``string`` is created before adding it. In both cases, ``name`` must
+            match ``[a-z_][a-z0-9_]*`` and cannot be a Python reserved word.
         value : float, optional
             Numerical Parameter value, typically the *initial value*.
         vary : bool, optional
             Whether the Parameter is varied during a fit (default is True).
         min : float, optional
-            Lower bound for value (default is `-numpy.inf`, no lower bound).
+            Lower bound for value (default is ``-numpy.inf``, no lower
+            bound).
         max : float, optional
-            Upper bound for value (default is `numpy.inf`, no upper bound).
+            Upper bound for value (default is ``numpy.inf``, no upper
+            bound).
         expr : str, optional
-            Mathematical expression used to constrain the value during the fit.
+            Mathematical expression used to constrain the value during the
+            fit (default is None).
         brute_step : float, optional
-            Step size for grid points in the `brute` method.
+            Step size for grid points in the `brute` method (default is
+            None).
 
         Examples
         --------
@@ -374,11 +421,11 @@ class Parameters(OrderedDict):
 
         Parameters
         ----------
-        parlist : :obj:`sequence` of :obj:`tuple` or :class:`Parameter`
-            A sequence of tuples, or a sequence of `Parameter` instances. If
-            it is a sequence of tuples, then each tuple must contain at least
-            the name. The order in each tuple must be `(name, value, vary,
-            min, max, expr, brute_step)`.
+        *parlist : :obj:`sequence` of :obj:`tuple` or Parameter
+            A sequence of tuples, or a sequence of `Parameter` instances.
+            If it is a sequence of tuples, then each tuple must contain at
+            least a `name`. The order in each tuple must be
+            ``(name, value, vary, min, max, expr, brute_step)``.
 
         Examples
         --------
@@ -410,12 +457,12 @@ class Parameters(OrderedDict):
 
         Returns
         -------
-        OrderedDict
-           An ordered dictionary of :attr:`name`::attr:`value` pairs for each
-           Parameter.
+        dict
+            A dictionary of :attr:`name`::attr:`value` pairs for each
+            Parameter.
 
         """
-        return OrderedDict((p.name, p.value) for p in self.values())
+        return {p.name: p.value for p in self.values()}
 
     def dumps(self, **kws):
         """Represent Parameters as a JSON string.
@@ -423,16 +470,16 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         **kws : optional
-            Keyword arguments that are passed to `json.dumps()`.
+            Keyword arguments that are passed to `json.dumps`.
 
         Returns
         -------
         str
-           JSON string representation of Parameters.
+            JSON string representation of Parameters.
 
         See Also
         --------
-        dump(), loads(), load(), json.dumps()
+        dump, loads, load, json.dumps
 
         """
         params = [p.__getstate__() for p in self.values()]
@@ -448,28 +495,30 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         **kws : optional
-            Keyword arguments that are passed to `json.loads()`.
+            Keyword arguments that are passed to `json.loads`.
 
         Returns
         -------
-        :class:`Parameters`
-           Updated Parameters from the JSON string.
+        Parameters
+            Updated Parameters from the JSON string.
 
         Notes
         -----
-        Current Parameters will be cleared before loading the data from the
-        JSON string.
+        Current Parameters will be cleared before loading the data from
+        the JSON string.
 
         See Also
         --------
-        dump(), dumps(), load(), json.loads()
+        dump, dumps, load, json.loads
 
         """
         self.clear()
 
-        tmp = decode4js(json.loads(s, **kws))
-        state = {'unique_symbols': tmp['unique_symbols'],
-                 'params': []}
+        tmp = json.loads(s, **kws)
+        unique_symbols = {key: decode4js(tmp['unique_symbols'][key]) for key
+                          in tmp['unique_symbols']}
+
+        state = {'unique_symbols': unique_symbols, 'params': []}
         for parstate in tmp['params']:
             _par = Parameter(name='')
             _par.__setstate__(parstate)
@@ -483,19 +532,19 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         fp : file-like object
-            An open and ``.write()``-supporting file-like object.
+            An open and `.write()`-supporting file-like object.
         **kws : optional
-            Keyword arguments that are passed to `dumps()`.
+            Keyword arguments that are passed to `dumps`.
 
         Returns
         -------
-        None or int
-            Return value from `fp.write()`. None for Python 2.7 and the
-            number of characters written in Python 3.
+        int
+            Return value from `fp.write()`: the number of characters
+            written.
 
         See Also
         --------
-        dump(), load(), json.dump()
+        dumps, load, json.dump
 
         """
         return fp.write(self.dumps(**kws))
@@ -506,35 +555,36 @@ class Parameters(OrderedDict):
         Parameters
         ----------
         fp : file-like object
-            An open and ``.read()``-supporting file-like object.
+            An open and `.read()`-supporting file-like object.
         **kws : optional
-            Keyword arguments that are passed to `loads()`.
+            Keyword arguments that are passed to `loads`.
 
         Returns
         -------
-        :class:`Parameters`
-           Updated Parameters loaded from `fp`.
+        Parameters
+            Updated Parameters loaded from `fp`.
 
         See Also
         --------
-        dump(), loads(), json.load()
+        dump, loads, json.load
 
         """
         return self.loads(fp.read(), **kws)
 
 
-class Parameter(object):
-    """A Parameter is an object that can be varied in a fit, or one of the
-    controlling variables in a model. It is a central component of lmfit,
-    and all minimization and modeling methods use Parameter objects.
+class Parameter:
+    """A Parameter is an object that can be varied in a fit.
+
+    It is a central component of lmfit, and all minimization and modeling
+    methods use Parameter objects.
 
     A Parameter has a `name` attribute, and a scalar floating point
-    `value`.  It also has a `vary` attribute that describes whether the
-    value should be varied during the minimization.  Finite bounds can be
+    `value`. It also has a `vary` attribute that describes whether the
+    value should be varied during the minimization. Finite bounds can be
     placed on the Parameter's value by setting its `min` and/or `max`
-    attributes.  A Parameter can also have its value determined by a
+    attributes. A Parameter can also have its value determined by a
     mathematical expression of other Parameter values held in the `expr`
-    attrribute.  Additional attributes include `brute_step` used as the step
+    attribute. Additional attributes include `brute_step` used as the step
     size in a brute-force minimization, and `user_data` reserved
     exclusively for user's need.
 
@@ -557,25 +607,30 @@ class Parameter(object):
         vary : bool, optional
             Whether the Parameter is varied during a fit (default is True).
         min : float, optional
-            Lower bound for value (default is `-numpy.inf`, no lower bound).
+            Lower bound for value (default is ``-numpy.inf``, no lower
+            bound).
         max : float, optional
-            Upper bound for value (default is `numpy.inf`, no upper bound).
+            Upper bound for value (default is ``numpy.inf``, no upper
+            bound).
         expr : str, optional
-            Mathematical expression used to constrain the value during the fit.
+            Mathematical expression used to constrain the value during the
+            fit (default is None).
         brute_step : float, optional
-            Step size for grid points in the `brute` method.
+            Step size for grid points in the `brute` method (default is
+            None).
         user_data : optional
-            User-definable extra attribute used for a Parameter.
+            User-definable extra attribute used for a Parameter (default
+            is None).
 
         Attributes
         ----------
         stderr : float
             The estimated standard error for the best-fit value.
         correl : dict
-            A dictionary of the correlation with the other fitted Parameters
-            of the form::
+            A dictionary of the correlation with the other fitted
+            Parameters of the form::
 
-            `{'decay': 0.404, 'phase': -0.020, 'frequency': 0.102}`
+            {'decay': 0.404, 'phase': -0.020, 'frequency': 0.102}
 
         """
         self.name = name
@@ -584,7 +639,7 @@ class Parameter(object):
         self.min = min
         self.max = max
         self.brute_step = brute_step
-        self.vary = vary
+        self._vary = vary
         self._expr = expr
         self._expr_ast = None
         self._expr_eval = None
@@ -597,7 +652,7 @@ class Parameter(object):
         self._init_bounds()
 
     def set(self, value=None, vary=None, min=None, max=None, expr=None,
-            brute_step=None):
+            brute_step=None, is_init_value=True):
         """Set or update Parameter attributes.
 
         Parameters
@@ -608,48 +663,46 @@ class Parameter(object):
             Whether the Parameter is varied during a fit.
         min : float, optional
             Lower bound for value. To remove a lower bound you must use
-            `-numpy.inf`.
+            ``-numpy.inf``.
         max : float, optional
             Upper bound for value. To remove an upper bound you must use
-            `numpy.inf`.
+            ``numpy.inf``.
         expr : str, optional
-            Mathematical expression used to constrain the value during the fit.
-            To remove a constraint you must supply an empty string.
+            Mathematical expression used to constrain the value during the
+            fit. To remove a constraint you must supply an empty string.
         brute_step : float, optional
             Step size for grid points in the `brute` method. To remove the
             step size you must use ``0``.
+        is_init_value: bool, optional
+            Whether to set value as `init_value`, when setting value.
 
         Notes
         -----
-        Each argument to `set()` has a default value of `None`, which will
-        leave the current value for the attribute unchanged.  Thus, to lift a
-        lower or upper bound, passing in `None` will not work.  Instead,
-        you must set these to `-numpy.inf` or `numpy.inf`, as with::
+        Each argument to `set()` has a default value of None, which will
+        leave the current value for the attribute unchanged. Thus, to lift
+        a lower or upper bound, passing in None will not work. Instead,
+        you must set these to ``-numpy.inf`` or ``numpy.inf``, as with::
 
             par.set(min=None)        # leaves lower bound unchanged
             par.set(min=-numpy.inf)  # removes lower bound
 
         Similarly, to clear an expression, pass a blank string, (not
-        ``None``!) as with::
+        None!) as with::
 
             par.set(expr=None)  # leaves expression unchanged
             par.set(expr='')    # removes expression
 
-        Explicitly setting a value or setting `vary=True` will also
+        Explicitly setting a value or setting ``vary=True`` will also
         clear the expression.
 
-        Finally, to clear the brute_step size, pass ``0``, not ``None``::
+        Finally, to clear the brute_step size, pass ``0``, not None::
 
             par.set(brute_step=None)  # leaves brute_step unchanged
             par.set(brute_step=0)     # removes brute_step
 
         """
-        if value is not None:
-            self.value = value
-            self.__set_expression('')
-
         if vary is not None:
-            self.vary = vary
+            self._vary = vary
             if vary:
                 self.__set_expression('')
 
@@ -659,6 +712,15 @@ class Parameter(object):
         if max is not None:
             self.max = max
 
+        # need to set this after min and max, so that it will use new
+        # bounds in the setter for value
+        if value is not None:
+            is_init_value = is_init_value or self.value in (None, -inf, inf)
+            self.value = value
+            if is_init_value:
+                self.init_value = value
+            self.__set_expression("")
+
         if expr is not None:
             self.__set_expression(expr)
 
@@ -680,7 +742,7 @@ class Parameter(object):
         if self.min > self.max:
             self.min, self.max = self.max, self.min
         if isclose(self.min, self.max, atol=1e-13, rtol=1e-13):
-            raise ValueError("Parameter '%s' has min == max" % self.name)
+            raise ValueError(f"Parameter '{self.name}' has min == max")
         if self._val > self.max:
             self._val = self.max
         if self._val < self.min:
@@ -689,41 +751,42 @@ class Parameter(object):
 
     def __getstate__(self):
         """Get state for pickle."""
-        return (self.name, self.value, self.vary, self.expr, self.min,
+        return (self.name, self.value, self._vary, self.expr, self.min,
                 self.max, self.brute_step, self.stderr, self.correl,
                 self.init_value, self.user_data)
 
     def __setstate__(self, state):
         """Set state for pickle."""
-        (self.name, _value, self.vary, self.expr, self.min, self.max,
+        (self.name, _value, self._vary, self.expr, self.min, self.max,
          self.brute_step, self.stderr, self.correl, self.init_value,
          self.user_data) = state
         self._expr_ast = None
         self._expr_eval = None
         self._expr_deps = []
         self._delay_asteval = False
-        self.value = _value
+        self._val = _value
         self._init_bounds()
+        self.value = _value
 
     def __repr__(self):
         """Return printable representation of a Parameter object."""
         s = []
-        sval = "value=%s" % repr(self._getval())
-        if not self.vary and self._expr is None:
+        sval = f"value={repr(self._getval())}"
+        if not self._vary and self._expr is None:
             sval += " (fixed)"
         elif self.stderr is not None:
-            sval += " +/- %.3g" % self.stderr
+            sval += f" +/- {self.stderr:.3g}"
         s.append(sval)
-        s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
+        s.append(f"bounds=[{repr(self.min)}:{repr(self.max)}]")
         if self._expr is not None:
-            s.append("expr='%s'" % self.expr)
+            s.append(f"expr='{self.expr}'")
         if self.brute_step is not None:
-            s.append("brute_step=%s" % (self.brute_step))
-        return "<Parameter '%s', %s>" % (self.name, ', '.join(s))
+            s.append(f"brute_step={self.brute_step}")
+        return f"<Parameter '{self.name}', {', '.join(s)}>"
 
     def setup_bounds(self):
-        """Set up Minuit-style internal/external parameter transformation of
-        min/max bounds.
+        """Set up Minuit-style internal/external parameter transformation
+        of min/max bounds.
 
         As a side-effect, this also defines the self.from_internal method
         used to re-calculate self.value from the internal value, applying
@@ -736,7 +799,7 @@ class Parameter(object):
         Returns
         -------
         _val : float
-            The internal value for parameter from self.value (which holds
+            The internal value for parameter from `self.value` (which holds
             the external, user-expected value). This internal value should
             actually be used in a fit.
 
@@ -758,6 +821,8 @@ class Parameter(object):
             self.from_internal = lambda val: self.min + (sin(val) + 1) * \
                                  (self.max - self.min) / 2.0
             _val = arcsin(2*(self._val - self.min)/(self.max - self.min) - 1)
+        if abs(_val) < tiny:
+            _val = 0.0
         return _val
 
     def scale_gradient(self, val):
@@ -765,7 +830,7 @@ class Parameter(object):
 
         Parameters
         ----------
-        val: float
+        val : float
             Numerical Parameter value.
 
         Returns
@@ -777,9 +842,9 @@ class Parameter(object):
         """
         if self.min == -inf and self.max == inf:
             return 1.0
-        elif self.max == inf:
+        if self.max == inf:
             return val / sqrt(val*val + 1)
-        elif self.min == -inf:
+        if self.min == -inf:
             return -val / sqrt(val*val + 1)
         return cos(val) * (self.max - self.min) / 2.0
 
@@ -790,33 +855,17 @@ class Parameter(object):
         # _expr_eval.symtable is kept up-to-date.
         # If you just assign to self._val then _expr_eval.symtable[self.name]
         # becomes stale if parameter.expr is not None.
-        if (isinstance(self._val, uncertainties.core.Variable) and
-                self._val is not nan):
-            msg = ("Please make sure that the Parameter value is a number, "
-                   "not an instance of 'uncertainties.core.Variable'. This "
-                   "automatic conversion will be removed in the next release.")
-            warnings.warn(FutureWarning(msg))
-            try:
-                self.value = self._val.nominal_value
-            except AttributeError:
-                pass
-
         if self._expr is not None:
             if self._expr_ast is None:
                 self.__set_expression(self._expr)
-            if self._expr_eval is not None:
-                if not self._delay_asteval:
-                    self.value = self._expr_eval(self._expr_ast)
-                    check_ast_errors(self._expr_eval)
+            if self._expr_eval is not None and not self._delay_asteval:
+                self.value = self._expr_eval(self._expr_ast)
+                check_ast_errors(self._expr_eval)
         return self._val
 
-    def set_expr_eval(self, evaluator):
-        """Set expression evaluator instance."""
-        self._expr_eval = evaluator
-
     @property
     def value(self):
-        """Return the numerical value of the Parameter, with bounds applied."""
+        """Return the numerical Parameter value, with bounds applied."""
         return self._getval()
 
     @value.setter
@@ -833,16 +882,26 @@ class Parameter(object):
         if self._expr_eval is not None:
             self._expr_eval.symtable[self.name] = self._val
 
+    @property
+    def vary(self):
+        """Return whether the parameter is variable"""
+        return self._vary
+
+    @vary.setter
+    def vary(self, val):
+        """Set whether a parameter is varied"""
+        self._vary = val
+        if val:
+            self.__set_expression('')
+
     @property
     def expr(self):
-        """Return the mathematical expression used to constrain the value
-        during the fit."""
+        """Return the mathematical expression used to constrain the value in fit."""
         return self._expr
 
     @expr.setter
     def expr(self, val):
-        """Set the mathematical expression used to constrain the value during
-        the fit.
+        """Set the mathematical expression used to constrain the value in fit.
 
         To remove a constraint you must supply an empty string.
 
@@ -854,7 +913,7 @@ class Parameter(object):
             val = None
         self._expr = val
         if val is not None:
-            self.vary = False
+            self._vary = False
         if not hasattr(self, '_expr_eval'):
             self._expr_eval = None
         if val is None:
@@ -889,7 +948,6 @@ class Parameter(object):
     def __bool__(self):
         """bool"""
         return self._getval() != 0
-    __nonzero__ = __bool__  # TODO: remove when PY3 only
 
     def __int__(self):
         """int"""
@@ -914,7 +972,6 @@ class Parameter(object):
     def __truediv__(self, other):
         """/"""
         return self._getval() / other
-    __div__ = __truediv__  # TODO: remove when PY3 only
 
     def __floordiv__(self, other):
         """//"""
@@ -967,7 +1024,6 @@ class Parameter(object):
     def __rtruediv__(self, other):
         """/ (right)"""
         return other / self._getval()
-    __rdiv__ = __rtruediv__  # TODO: remove when PY3 only
 
     def __rdivmod__(self, other):
         """divmod (right)"""
@@ -994,9 +1050,35 @@ class Parameter(object):
         return other - self._getval()
 
 
-def isParameter(x):
-    """Test for Parameter-ness."""
-    msg = 'The isParameter function will be removed in the next release.'
-    warnings.warn(FutureWarning(msg))
-    return (isinstance(x, Parameter) or
-            x.__class__.__name__ == 'Parameter')
+def create_params(**kws):
+    """Create lmfit.Parameters instance and set initial values and attributes.
+
+    Parameters
+    ----------
+    **kws
+        keywords are parameter names, value are dictionaries of Parameter
+        values and attributes.
+
+    Returns
+    -------
+    Parameters instance
+
+    Notes
+    -----
+    1. keyword arguments will be used to create parameter names.
+    2. values can either be numbers (floats or integers) to set the parameter
+       value, or can be dictionaries with any of the following keywords:
+       ``value``, ``vary``, ``min``, ``max``, ``expr``, ``brute_step``, or
+       ``is_init_value`` to set those parameter attributes.
+    3. for each parameter,  ``is_init_value`` controls whether to set
+       ``init_value`` when setting ``value``, and defaults to True.
+
+    Examples
+    --------
+    >>> params = create_params(amplitude=2, center=200,
+                               sigma={'value': 3, 'min':0},
+                               fwhm={'expr': '2.0*sigma'})
+    """
+    params = Parameters()
+    params.set(**kws)
+    return params
