@@ -1,12 +1,13 @@
 """Functions to display fitting results and confidence intervals."""
+
 from math import log10
 import re
+
 import numpy as np
 
 try:
-    import numdifftools
+    import numdifftools  # noqa: F401
     HAS_NUMDIFFTOOLS = True
-    flake8_is_wrong_but_easily_foold = numdifftools.Hessian
 except ImportError:
     HAS_NUMDIFFTOOLS = False
 
@@ -22,9 +23,9 @@ def getfloat_attr(obj, attr, length=11):
     val = getattr(obj, attr, None)
     if val is None:
         return 'unknown'
-    elif isinstance(val, int):
-        return '%d' % val
-    elif isinstance(val, float):
+    if isinstance(val, int):
+        return f'{val}'
+    if isinstance(val, float):
         return gformat(val, length=length).strip()
     return repr(val)
 
@@ -38,14 +39,14 @@ def gformat(val, length=11):
         b) the precision will be as high as possible.
         c) trailing zeros will not be trimmed.
 
-    The precision will typically be length-7.
+    The precision will typically be ``length-7``.
 
     Parameters
     ----------
     val : float
-       Value to be formatted.
+        Value to be formatted.
     length : int, optional
-       Length of output string (default is 11).
+        Length of output string (default is 11).
 
     Returns
     -------
@@ -54,13 +55,18 @@ def gformat(val, length=11):
 
     Notes
     ------
-     Positive values will have leading blank.
+    Positive values will have leading blank.
 
     """
+    if val is None or isinstance(val, bool):
+        return f'{repr(val):>{length}s}'
     try:
         expon = int(log10(abs(val)))
     except (OverflowError, ValueError):
         expon = 0
+    except TypeError:
+        return f'{repr(val):>{length}s}'
+
     length = max(length, 7)
     form = 'e'
     prec = length - 7
@@ -72,15 +78,11 @@ def gformat(val, length=11):
         prec += 4
         if expon > 0:
             prec -= expon
-    fmt = '{0: %i.%i%s}' % (length, prec, form)
-    return fmt.format(val)[:length]
-
-
-CORREL_HEAD = '[[Correlations]] (unreported correlations are < %.3f)'
+    return f'{val:{length}.{prec}{form}}'
 
 
 def fit_report(inpars, modelpars=None, show_correl=True, min_correl=0.1,
-               sort_pars=False):
+               sort_pars=False, correl_mode='list'):
     """Generate a report of the fitting results.
 
     The report contains the best-fit values for the parameters and their
@@ -88,25 +90,30 @@ def fit_report(inpars, modelpars=None, show_correl=True, min_correl=0.1,
 
     Parameters
     ----------
-    inpars  : Parameters
-       Input Parameters from fit or MinimizerResult returned from a fit.
+    inpars : Parameters
+        Input Parameters from fit or MinimizerResult returned from a fit.
     modelpars : Parameters, optional
-       Known Model Parameters.
+        Known Model Parameters.
     show_correl : bool, optional
-       Whether to show list of sorted correlations (default is True).
+        Whether to show list of sorted correlations (default is True).
     min_correl : float, optional
-       Smallest correlation in absolute value to show (default is 0.1).
+        Smallest correlation in absolute value to show (default is 0.1).
     sort_pars : bool or callable, optional
-       Whether to show parameter names sorted in alphanumerical order. If
-       False (default), then the parameters will be listed in the order they
-       were added to the Parameters dictionary. If callable, then this (one
-       argument) function is used to extract a comparison key from each
-       list element.
+        Whether to show parameter names sorted in alphanumerical order. If
+        False (default), then the parameters will be listed in the order
+        they were added to the Parameters dictionary. If callable, then
+        this (one argument) function is used to extract a comparison key
+        from each list element.
+    correl_mode : {'list', table'} str, optional
+        Mode for how to show correlations. Can be either 'list' (default)
+        to show a sorted (if ``sort_pars`` is True) list of correlation
+        values, or 'table' to show a complete, formatted table of
+        correlations.
 
     Returns
     -------
-    string
-       Multi-line text of fit report.
+    str
+        Multi-line text of fit report.
 
     """
     from .parameter import Parameters
@@ -129,64 +136,71 @@ def fit_report(inpars, modelpars=None, show_correl=True, min_correl=0.1,
 
     buff = []
     add = buff.append
-    namelen = max([len(n) for n in parnames])
+    namelen = max(len(n) for n in parnames)
     if result is not None:
         add("[[Fit Statistics]]")
-        add("    # fitting method   = %s" % (result.method))
-        add("    # function evals   = %s" % getfloat_attr(result, 'nfev'))
-        add("    # data points      = %s" % getfloat_attr(result, 'ndata'))
-        add("    # variables        = %s" % getfloat_attr(result, 'nvarys'))
-        add("    chi-square         = %s" % getfloat_attr(result, 'chisqr'))
-        add("    reduced chi-square = %s" % getfloat_attr(result, 'redchi'))
-        add("    Akaike info crit   = %s" % getfloat_attr(result, 'aic'))
-        add("    Bayesian info crit = %s" % getfloat_attr(result, 'bic'))
+        add(f"    # fitting method   = {result.method}")
+        add(f"    # function evals   = {getfloat_attr(result, 'nfev')}")
+        add(f"    # data points      = {getfloat_attr(result, 'ndata')}")
+        add(f"    # variables        = {getfloat_attr(result, 'nvarys')}")
+        add(f"    chi-square         = {getfloat_attr(result, 'chisqr')}")
+        add(f"    reduced chi-square = {getfloat_attr(result, 'redchi')}")
+        add(f"    Akaike info crit   = {getfloat_attr(result, 'aic')}")
+        add(f"    Bayesian info crit = {getfloat_attr(result, 'bic')}")
+        if hasattr(result, 'rsquared'):
+            add(f"    R-squared          = {getfloat_attr(result, 'rsquared')}")
         if not result.errorbars:
             add("##  Warning: uncertainties could not be estimated:")
             if result.method in ('leastsq', 'least_squares') or HAS_NUMDIFFTOOLS:
-                for name in parnames:
+                parnames_varying = [par for par in result.params
+                                    if result.params[par].vary]
+                for name in parnames_varying:
                     par = params[name]
                     space = ' '*(namelen-len(name))
-                    if np.allclose(par.value, par.init_value):
-                        add('    %s:%s  at initial value' % (name, space))
-                    if (np.allclose(par.value, par.min) or np.allclose(par.value, par.min)):
-                        add('    %s:%s  at boundary' % (name, space))
+                    if par.init_value and np.allclose(par.value, par.init_value):
+                        add(f'    {name}:{space}  at initial value')
+                    if (np.allclose(par.value, par.min) or np.allclose(par.value, par.max)):
+                        add(f'    {name}:{space}  at boundary')
             else:
                 add("    this fitting method does not natively calculate uncertainties")
-                add("    and numdifftools is not installed for lmfit to do this.  Use")
+                add("    and numdifftools is not installed for lmfit to do this. Use")
                 add("    `pip install numdifftools` for lmfit to estimate uncertainties")
                 add("    with this fitting method.")
 
-    namelen = max([len(n) for n in parnames])
     add("[[Variables]]")
     for name in parnames:
         par = params[name]
         space = ' '*(namelen-len(name))
-        nout = "%s:%s" % (name, space)
+        nout = f"{name}:{space}"
         inval = '(init = ?)'
         if par.init_value is not None:
-            inval = '(init = %.7g)' % par.init_value
+            inval = f'(init = {par.init_value:.7g})'
         if modelpars is not None and name in modelpars:
-            inval = '%s, model_value = %.7g' % (inval, modelpars[name].value)
+            inval = f'{inval}, model_value = {modelpars[name].value:.7g}'
         try:
             sval = gformat(par.value)
         except (TypeError, ValueError):
-            sval = 'Non Numeric Value?'
+            sval = ' Non Numeric Value?'
         if par.stderr is not None:
             serr = gformat(par.stderr)
             try:
-                spercent = '({:.2%})'.format(abs(par.stderr/par.value))
+                spercent = f'({abs(par.stderr/par.value):.2%})'
             except ZeroDivisionError:
                 spercent = ''
-            sval = '%s +/-%s %s' % (sval, serr, spercent)
+            sval = f'{sval} +/-{serr} {spercent}'
 
         if par.vary:
-            add("    %s %s %s" % (nout, sval, inval))
+            add(f"    {nout} {sval} {inval}")
         elif par.expr is not None:
-            add("    %s %s == '%s'" % (nout, sval, par.expr))
+            add(f"    {nout} {sval} == '{par.expr}'")
         else:
-            add("    %s % .7g (fixed)" % (nout, par.value))
+            add(f"    {nout} {par.value: .7g} (fixed)")
 
-    if show_correl:
+    if show_correl and correl_mode.startswith('tab'):
+        add('[[Correlations]] ')
+        for line in correl_table(params).split('\n'):
+            buff.append('  %s' % line)
+    elif show_correl:
         correls = {}
         for i, name in enumerate(parnames):
             par = params[name]
@@ -196,26 +210,45 @@ def fit_report(inpars, modelpars=None, show_correl=True, min_correl=0.1,
                 for name2 in parnames[i+1:]:
                     if (name != name2 and name2 in par.correl and
                             abs(par.correl[name2]) > min_correl):
-                        correls["%s, %s" % (name, name2)] = par.correl[name2]
+                        correls[f"{name}, {name2}"] = par.correl[name2]
 
         sort_correl = sorted(correls.items(), key=lambda it: abs(it[1]))
         sort_correl.reverse()
         if len(sort_correl) > 0:
-            add(CORREL_HEAD % min_correl)
-            maxlen = max([len(k) for k in list(correls.keys())])
+            add('[[Correlations]] (unreported correlations are < '
+                f'{min_correl:.3f})')
+            maxlen = max(len(k) for k in list(correls.keys()))
         for name, val in sort_correl:
             lspace = max(0, maxlen - len(name))
-            add('    C(%s)%s = % .3f' % (name, (' '*30)[:lspace], val))
+            add(f"    C({name}){(' '*30)[:lspace]} = {val:+.4f}")
     return '\n'.join(buff)
 
 
 def fitreport_html_table(result, show_correl=True, min_correl=0.1):
-    """Report minimizer result as an html table"""
+    """Generate a report of the fitting result as an HTML table.
+
+    Parameters
+    ----------
+    result : MinimizerResult or ModelResult
+        Object containing the optimized parameters and several
+        goodness-of-fit statistics.
+    show_correl : bool, optional
+        Whether to show list of sorted correlations (default is True).
+    min_correl : float, optional
+        Smallest correlation in absolute value to show (default is 0.1).
+
+    Returns
+    -------
+    str
+        Multi-line HTML code of fit report.
+
+    """
     html = []
     add = html.append
 
     def stat_row(label, val, val2=''):
-        add('<tr><td>%s</td><td>%s</td><td>%s</td></tr>' % (label, val, val2))
+        add(f'<tr><td>{label}</td><td>{val}</td><td>{val2}</td></tr>')
+
     add('<h2>Fit Statistics</h2>')
     add('<table>')
     stat_row('fitting method', result.method)
@@ -226,9 +259,11 @@ def fitreport_html_table(result, show_correl=True, min_correl=0.1):
     stat_row('reduced chi-square', gformat(result.redchi))
     stat_row('Akaike info crit.', gformat(result.aic))
     stat_row('Bayesian info crit.', gformat(result.bic))
+    if hasattr(result, 'rsquared'):
+        stat_row('R-squared', gformat(result.rsquared))
     add('</table>')
     add('<h2>Variables</h2>')
-    add(result.params._repr_html_())
+    add(params_html_table(result.params))
     if show_correl:
         correls = []
         parnames = list(result.params.keys())
@@ -244,26 +279,74 @@ def fitreport_html_table(result, show_correl=True, min_correl=0.1):
         if len(correls) > 0:
             sort_correls = sorted(correls, key=lambda val: abs(val[2]))
             sort_correls.reverse()
-            extra = '(unreported correlations are < %.3f)' % (min_correl)
-            add('<h2>Correlations %s</h2>' % extra)
+            extra = f'(unreported correlations are < {min_correl:.3f})'
+            add(f'<h2>Correlations {extra}</h2>')
             add('<table>')
             for name1, name2, val in sort_correls:
-                stat_row(name1, name2, "%.4f" % val)
+                stat_row(name1, name2, f"{val:+.4f}")
             add('</table>')
     return ''.join(html)
 
 
+def correl_table(params):
+    """Return a printable correlation table for a Parameters object."""
+    varnames = [vname for vname in params if params[vname].vary]
+    nwid = max(8, max([len(vname) for vname in varnames])) + 1
+
+    def sfmt(a):
+        return f" {a:{nwid}s}"
+
+    def ffmt(a):
+        return sfmt(f"{a:+.4f}")
+
+    title = ['', sfmt('Variable')]
+    title.extend([sfmt(vname) for vname in varnames])
+
+    title = '|'.join(title) + '|'
+    bar = [''] + ['-'*(nwid+1) for i in range(len(varnames)+1)] + ['']
+    bar = '+'.join(bar)
+
+    buff = [bar, title, bar]
+
+    for vname, par in params.items():
+        if not par.vary:
+            continue
+        line = ['', sfmt(vname)]
+        for vother in varnames:
+            if vother == vname:
+                line.append(ffmt(1))
+            elif vother in par.correl:
+                line.append(ffmt(par.correl[vother]))
+            else:
+                line.append('unknown')
+        buff.append('|'.join(line) + '|')
+    buff.append(bar)
+    return '\n'.join(buff)
+
+
 def params_html_table(params):
-    """Returns a HTML representation of parameters data."""
-    has_err = any([p.stderr is not None for p in params.values()])
-    has_expr = any([p.expr is not None for p in params.values()])
-    has_brute = any([p.brute_step is not None for p in params.values()])
+    """Return an HTML representation of Parameters.
+
+    Parameters
+    ----------
+    params : Parameters
+        Object containing the Parameters of the model.
+
+    Returns
+    -------
+    str
+        Multi-line HTML code of fitting parameters.
+
+    """
+    has_err = any(p.stderr is not None for p in params.values())
+    has_expr = any(p.expr is not None for p in params.values())
+    has_brute = any(p.brute_step is not None for p in params.values())
 
     html = []
     add = html.append
 
     def cell(x, cat='td'):
-        return add('<%s> %s </%s>' % (cat, x, cat))
+        return add(f'<{cat}> {x} </{cat}>')
 
     add('<table><tr>')
     headers = ['name', 'value']
@@ -278,26 +361,29 @@ def params_html_table(params):
         cell(h, cat='th')
     add('</tr>')
 
-    for p in params.values():
-        rows = [p.name, gformat(p.value)]
+    for par in params.values():
+        rows = [par.name, gformat(par.value)]
         if has_err:
-            serr, perr = '', ''
-            if p.stderr is not None:
-                serr = gformat(p.stderr)
-                perr = '%.2f%%' % (100 * p.stderr / p.value)
-            rows.extend([serr, perr])
-        rows.extend((p.init_value, gformat(p.min), gformat(p.max),
-                     '%s' % p.vary))
+            serr = ''
+            spercent = ''
+            if par.stderr is not None:
+                serr = gformat(par.stderr)
+                try:
+                    spercent = f'({abs(par.stderr/par.value):.2%})'
+                except ZeroDivisionError:
+                    pass
+            rows.extend([serr, spercent])
+        rows.extend((par.init_value, gformat(par.min),
+                     gformat(par.max), f'{par.vary}'))
         if has_expr:
             expr = ''
-            if p.expr is not None:
-                expr = p.expr
+            if par.expr is not None:
+                expr = par.expr
             rows.append(expr)
-
         if has_brute:
             brute_step = 'None'
-            if p.brute_step is not None:
-                brute_step = gformat(p.brute_step)
+            if par.brute_step is not None:
+                brute_step = gformat(par.brute_step)
             rows.append(brute_step)
 
         add('<tr>')
@@ -308,13 +394,8 @@ def params_html_table(params):
     return ''.join(html)
 
 
-def report_errors(params, **kws):
-    """Print a report for fitted params: see error_report()."""
-    print(fit_report(params, **kws))
-
-
 def report_fit(params, **kws):
-    """Print a report for fitted params: see error_report()."""
+    """Print a report of the fitting results."""
     print(fit_report(params, **kws))
 
 
@@ -323,26 +404,30 @@ def ci_report(ci, with_offset=True, ndigits=5):
 
     Parameters
     ----------
+    ci : dict
+        The result of :func:`~lmfit.confidence.conf_interval`: a dictionary
+        containing a list of ``(sigma, vals)``-tuples for each parameter.
     with_offset : bool, optional
-        Whether to subtract best value from all other values (default is True).
+        Whether to subtract best value from all other values (default is
+        True).
     ndigits : int, optional
         Number of significant digits to show (default is 5).
 
     Returns
     -------
     str
-       Text of formatted report on confidence intervals.
+        Text of formatted report on confidence intervals.
 
     """
-    maxlen = max([len(i) for i in ci])
+    maxlen = max(len(i) for i in ci)
     buff = []
     add = buff.append
 
     def convp(x):
-        """TODO: function docstring."""
+        """Convert probabilities into header for CI report."""
         if abs(x[0]) < 1.e-2:
             return "_BEST_"
-        return "%.2f%%" % (x[0]*100)
+        return f"{x[0] * 100:.2f}%"
 
     title_shown = False
     fmt_best = fmt_diff = "{0:.%if}" % ndigits
@@ -353,7 +438,7 @@ def ci_report(ci, with_offset=True, ndigits=5):
             add("".join([''.rjust(maxlen+1)] + [i.rjust(ndigits+5)
                                                 for i in map(convp, row)]))
             title_shown = True
-        thisrow = [" %s:" % name.ljust(maxlen)]
+        thisrow = [f" {name.ljust(maxlen)}:"]
         offset = 0.0
         if with_offset:
             for cval, val in row:
